<?php
/**
 * Created by PhpStorm.
 * User: phillipesilva
 * Date: 07/08/14
 * Time: 15:17
 */

namespace Folha\Logic\ProgramacaoOcorrencia;

use Folha\Entities\GpeAlteescala;
use Folha\Entities\GpeEscalatrabm;
use Folha\Entities\GpeHorarioturno;
use Folha\Entities\GpeHordiah;
use Folha\Entities\GpeHordiam;
use Folha\Entities\GpeTurno;
use Folha\Entities\PtoProgocorrencia;
use Folha\Entities\PtoTipoprogocorrencia;
use HCM\Util\DateUtil;
use Geral\Util\NovoCodigo;
use Setforms2\Shared\Registry;

class ProgramacaoOcorrencia
{
    const IDENTIFICA_INTERVALO = 'IDENTIFICA_INTERVALO';

    public static function cadastraProgramacoesOcorrencia($dados)
    {
        $entityManager = Registry::getEntityManager();
        $db = $entityManager->getConnection();
        $db->beginTransaction();

        try {
            foreach ($dados['VINCULOS'] as $vinculo) {
                $progOcorr = new PtoProgocorrencia();
                $progOcorr->setNrprogocorrencia(NovoCodigo::geraCodigo($dados['NRORG'], 'PTO_PROGOCORRENCIA', 1));
                $progOcorr->setNrmotivoocorfreq($dados['NRMOTIVOOCORFREQ']);
                $progOcorr->setNrhordiam($dados['NRHORDIAM']);
                $progOcorr->setNrsitufrequencia($dados['NRSITUFREQUENCIA']);
                $progOcorr->setNrvinculom($vinculo['NRVINCULOM']);
                $progOcorr->setNrtipoprogocorrencia($dados['NRTIPOPROGOCORRENCIA']);
                $progOcorr->setDtinicioprogocor($dados['DTINICIOPROGOCOR']);
                $progOcorr->setDtfimprogocor($dados['DTFIMPROGOCOR']);
                $progOcorr->setIddiassemana($dados['IDDIASSEMANA']);
                $progOcorr->setIdtipodiacalendario($dados['IDTIPODIACALENDARIO']);
                $progOcorr->setIdtipohorario($dados['IDTIPOHORARIO']);
                $progOcorr->setNrsitufreqgatilho($dados['NRSITUFREQGATILHO']);
                $progOcorr->setQtmingatilhoprogocor($dados['QTMINGATILHOPROGOCOR']);
                $progOcorr->setQtmaxgatilhoprogocor($dados['QTMAXGATILHOPROGOCOR']);
                $progOcorr->setVrcoeficienteacrescimo($dados['VRCOEFICIENTEACRESCIMO']);
                $progOcorr->setIdativo('S');
                $progOcorr->setNrorg($dados['NRORG']);
                $progOcorr->setNrorginclusao($dados['NRORG']);
                $progOcorr->setNrorgultatu($dados['NRORG']);
                $progOcorr->setCdoperinclusao($dados['CDOPERADOR']);
                $progOcorr->setCdoperultatu($dados['CDOPERADOR']);
                $progOcorr->setDtinclusao($dados['DATA_ATUAL']);
                $progOcorr->setDtultatu($dados['DATA_ATUAL']);
                $progOcorr->persist();
            }

            $entityManager->flush();
            $db->commit();
        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }
    }

    public static function atualizaIdDiasSemana($nrProg, $nrOrg, $idDiasSemana)
    {
        $entityManager = Registry::getEntityManager();
        $db = $entityManager->getConnection();
        $db->beginTransaction();

        try {
            $programacao = PtoProgocorrencia::findOneByNrprogocorrenciaAndNrorg($nrProg, $nrOrg);

            if (is_object($programacao)) {
                $programacao->setIddiassemana($idDiasSemana);
                $programacao->persist();
            }

            $entityManager->flush();
            $db->commit();
        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }
    }

    public static function retornaHorarioHistorico($nrOrgTrab, $nrHorDiaM, $dtCompetencia)
    {
        $hordiah = GpeHordiah::getObjectByWhere("NRORG = ?
                                                 AND NRHORDIAM = ?
                                                 AND DTMESCOMPETENC IN (SELECT MAX(DTMESCOMPETENC)
                                                                          FROM GPE_HORDIAH
                                                                         WHERE NRORG = ?
                                                                           AND NRHORDIAM = ?
                                                                           AND DTMESCOMPETENC <= TO_DATE(?, 'DD/MM/YYYY'))",
            array($nrOrgTrab, $nrHorDiaM, $nrOrgTrab, $nrHorDiaM, $dtCompetencia));

        return $hordiah;
    }

    public static function retornaHorarioHistoricoData($nrOrgTrab, $nrVinculo, $dtSeguinte)
    {
        $where = "NRORG = ?
               AND NRVINCULOM = ?
               AND TO_DATE(?,'DD/MM/YYYY')
               BETWEEN DTINIESCALA
               AND NVL(DTFIMESCALA, TO_DATE(?, 'DD/MM/YYYY'))";
        $params = array(
            $nrOrgTrab,
            $nrVinculo,
            $dtSeguinte->format(DateUtil::FORMATO_BRASILEIRO),
            $dtSeguinte->format(DateUtil::FORMATO_BRASILEIRO)
        );
        $orderBy = "DTINIESCALA DESC";

        /** @var GpeAlteescala $alteEscala */
        $alteEscala = GpeAlteescala::getObjectByWhere($where, $params, $orderBy);

        if (is_object($alteEscala)) {
            $escala = GpeEscalatrabm::findOneByNrescalatrabmAndNrorg($alteEscala->getNrescalatrabm(), $nrOrgTrab);

            $nrHorDiaM = $alteEscala->getNrhordiam();

            if (is_object($escala)) {
                $turno = $alteEscala->getTurno();
                if (is_object($turno)) {
                    if(is_null($nrHorDiaM) || $nrHorDiaM == ''){
                        //Verifica nas Programações de Ocorrência
                        $nrOrgPadrao = \FPA::getNrorgPadrao();
                        /** @var PtoTipoprogocorrencia $tipoProgOcor */
                        $tipoProgOcor = PtoTipoprogocorrencia::findOneByNrorgAndCdtipoprogocorrencia($nrOrgPadrao,
                                                                                            self::IDENTIFICA_INTERVALO);

                        $whereProgForcaHorario = "NRVINCULOM = ?
                                  AND NRORG = ?
                                  AND TO_DATE(?, 'DD/MM/YYYY') BETWEEN DTINICIOPROGOCOR AND NVL(DTFIMPROGOCOR, TO_DATE(?, 'DD/MM/YYYY'))
                                  AND NRHORDIAM IS NOT NULL
                                  AND DTINICIOPROGOCOR IN (SELECT MAX(DTINICIOPROGOCOR)
                                                             FROM PTO_PROGOCORRENCIA
                                                            WHERE NRVINCULOM = ?
                                                              AND NRORG = ?
                                                              AND NRHORDIAM IS NOT NULL
                                                              AND TO_DATE(?, 'DD/MM/YYYY') BETWEEN DTINICIOPROGOCOR
                                                                  AND NVL(DTFIMPROGOCOR, TO_DATE(?, 'DD/MM/YYYY')))
                                  AND NRTIPOPROGOCORRENCIA <> ?";
                        $paramsProgForcaHorario = array(
                            $nrVinculo,
                            $nrOrgTrab,
                            $dtSeguinte->format(DateUtil::FORMATO_BRASILEIRO),
                            $dtSeguinte->format(DateUtil::FORMATO_BRASILEIRO),
                            $nrVinculo,
                            $nrOrgTrab,
                            $dtSeguinte->format(DateUtil::FORMATO_BRASILEIRO),
                            $dtSeguinte->format(DateUtil::FORMATO_BRASILEIRO),
                            $tipoProgOcor->getNrtipoprogocorrencia()
                        );

                        /** @var PtoProgocorrencia $progForcaHorario */
                        $progForcaHorario = PtoProgocorrencia::getObjectByWhere($whereProgForcaHorario, $paramsProgForcaHorario);

                        if(is_object($progForcaHorario)){
                            $horario = GpeHordiam::findOneByNrorgAndNrhordiam($nrOrgTrab, $progForcaHorario->getNrhordiam());
                        }
                        else{
                            /** @var GpeTurno $turno*/
                            $horario = self::getHorarioTurno($dtSeguinte,
                                $turno->getNrturno(),
                                $turno->getDtdatabase(),
                                $turno->getNrorg());
                        }

                        $nrHorDiaM = $horario->getNrhordiam();
                    }

                    $horDiaH = self::retornaHorarioHistorico($nrOrgTrab, $nrHorDiaM, \FPA::getDtCompetencia());

                    return $horDiaH;
                }
            }
        }
    }

    public static function getHorarioTurno($data, $nrTurno, $dtDataBase, $nrOrg)
    {
        $diaHorario = DateUtil::getQtdeDiasEntre($dtDataBase, $data);

        $where   = 'NRTURNO = ? AND NRORG = ?';
        $params  = array($nrTurno, $nrOrg);
        $orderBy = 'NRSEQHORARIO';

        $horariosTurno = GpeHorarioturno::getObjectsByWhere($where, $params, $orderBy);
        $diaHorario    = fmod($diaHorario, count($horariosTurno));
        $horarioTurno  = $horariosTurno[$diaHorario];

        return $horarioTurno;
    }
}